/*
  LCD2USB control module

  T. Nakagawa
  2011/04/15
*/


#include "clcd.h"
#include <usb.h>


#define LCD2USB_VID 0x0403
#define LCD2USB_PID 0xc630
#define TIMEOUT 1000
#define CMD_ECHO (0 << 5)
#define CMD_CMD  (1 << 5)
#define CMD_DATA (2 << 5)
#define CMD_SET  (3 << 5)
#define CMD_GET  (4 << 5)
#define TGT_CTRL0 (1 << 3)
#define TGT_CTRL1 (1 << 4)
#define SET_CONTRAST   (0 << 3)
#define SET_BRIGHTNESS (1 << 3)
#define GET_FWVER (0 << 3)
#define GET_KEYS  (1 << 3)
#define GET_CTRL  (2 << 3)


char *clcd_error;
static usb_dev_handle *Handle = NULL;


int clcd_open(void) {
  struct usb_bus *bus;
  struct usb_device *dev;

  usb_init();
  usb_find_busses();
  usb_find_devices();

  for (bus = usb_get_busses(); bus != NULL; bus = bus->next) {
    for (dev = bus->devices; dev != NULL; dev = dev->next) {
      if (dev->descriptor.idVendor == LCD2USB_VID && dev->descriptor.idProduct == LCD2USB_PID) {
	if ((Handle = usb_open(dev)) == NULL) {
	  clcd_error = usb_strerror();
	  return 1;
	}
      }
    }
  }
  if (Handle == NULL) {
    clcd_error = "cannot find LCD2USB device";
    return 2;
  }

  return 0;
}


void clcd_close(void) {
  usb_close(Handle);

  return;
}


void clcd_setcontrast(unsigned char val) {
  usb_control_msg(Handle, USB_TYPE_VENDOR, (CMD_SET | SET_CONTRAST), val, 0, NULL, 0, TIMEOUT);

  return;
}


void clcd_setbrightness(unsigned char val) {
  usb_control_msg(Handle, USB_TYPE_VENDOR, (CMD_SET | SET_BRIGHTNESS), val, 0, NULL, 0, TIMEOUT);

  return;
}


void clcd_print(const char *str) {
  int i, j;
  unsigned int idx, val;

  for (i = 0; i < 2; i++) {
    usb_control_msg(Handle, USB_TYPE_VENDOR, (CMD_CMD | TGT_CTRL0), (0x80 | (0x40 * i)), 0, NULL, 0, TIMEOUT);  /* Set DDRAM Address */
    for (j = 0; j < CLCD_WIDTH / 4; j++) {
      val = (unsigned int)*str++;
      val |= (unsigned int)*str++ << 8;
      idx = (unsigned int)*str++;
      idx |= (unsigned int)*str++ << 8;
      usb_control_msg(Handle, USB_TYPE_VENDOR, (CMD_DATA | TGT_CTRL0 | 3), val, idx, NULL, 0, TIMEOUT);
    }
  }

  return;
}


unsigned char clcd_getkey(void) {
  unsigned char buf[2];

  if (usb_control_msg(Handle, (USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN), (CMD_GET | GET_KEYS), 0, 0, (char *)buf, sizeof(buf), TIMEOUT) < 0) {
    return 0;
  }

  return buf[0];
}
