/*
  Server monitor daemon

  T. Nakagawa
  2011/04/16
*/


#include <stdio.h>
#include <time.h>
#include <unistd.h>


#define SLEEP 3	/* Update interval [sec] */


double get_hddtemp(void) {
  const int interval = 60;
  static time_t lastupd = 0;
  static double temp = 0.0;
  FILE *fp;
  unsigned short id, val;
  time_t t;

  if (time(&t) - lastupd >= interval) {
    temp = 0.0;
    if ((fp = fopen("/proc/ide/hda/smart_values", "r")) == NULL) return 0.0;
    fscanf(fp, "%*x");
    while (fscanf(fp, "%hx %*x %hx", &id, &val) == 2) {
      if ((id & 0xff) == 0xc2) {	/* Temperature */
	temp = (double)(val >> 8);
	break;
      }
      fscanf(fp, "%*x %*x %*x");
    }
    fclose(fp);
    lastupd = t;
  }

  return temp;
}


double get_cputemp(void) {
  FILE *fp;
  int v;
  double temp;

  if ((fp = fopen("/sys/devices/platform/vt8231.24576/temp1_input", "r")) == NULL) return 0.0;
  fscanf(fp, "%d", &v);
  fclose(fp);
  temp = v / 781.0 - 57.6;	/* for EPIA-5000 */

  return temp;
}


double get_cpuload(void) {
  FILE *fp;
  double load;

  if ((fp = fopen("/proc/loadavg", "r")) == NULL) return 0.0;
  fscanf(fp, "%lf", &load);
  fclose(fp);

  return load;
}


int get_freemem(void) {
  FILE *fp;
  int mem;

  mem = 0;
  if ((fp = fopen("/proc/meminfo", "r")) == NULL) return 0.0;
  fscanf(fp, "%*s %*d %*s MemFree: %d kB", &mem);	/* Free memory in kB */
  fclose(fp);

  return mem;
}


int main(void) {
  for (; ; ) {
    double cpuload;
    int freemem;
    double cputemp;
    double hddtemp;

    cpuload = get_cpuload();
    freemem = get_freemem();
    cputemp = get_cputemp();
    hddtemp = get_hddtemp();

    printf("Load:%4.2f CPU:%2dMem:%4dM HDD:%2d\n", cpuload, (int)(cputemp + 0.5), (int)(freemem / 1024 + 0.5), (int)(hddtemp + 0.5));
    fflush(stdout);

    sleep(SLEEP);
  }

  return 0;
}
